==> An_V1.9.0.tar.gz: kmpdfkit_demo/Forms/src/main/java/com/compdfkit/forms/PDFFormSampleActivity.java <==
package com.compdfkit.forms;

import android.content.Intent;
import android.graphics.Paint;
import android.net.Uri;
import android.os.Bundle;
import android.view.View;

import androidx.activity.result.ActivityResultLauncher;
import androidx.activity.result.contract.ActivityResultContracts;
import androidx.annotation.NonNull;
import androidx.core.content.ContextCompat;

import com.compdfkit.core.annotation.CPDFAnnotation;
import com.compdfkit.core.annotation.form.CPDFWidget;
import com.compdfkit.forms.databinding.FormPdfSampleActivityBinding;
import com.compdfkit.tools.common.activity.CBasicActivity;
import com.compdfkit.tools.common.utils.CFileUtils;
import com.compdfkit.tools.common.utils.annotation.CPDFAnnotationManager;
import com.compdfkit.tools.common.utils.dialog.CAlertDialog;
import com.compdfkit.tools.common.utils.task.CExtractAssetFileTask;
import com.compdfkit.tools.common.utils.window.CPopupMenuWindow;
import com.compdfkit.tools.common.views.pdfbota.CPDFBOTA;
import com.compdfkit.tools.common.views.pdfbota.CPDFBotaDialogFragment;
import com.compdfkit.tools.common.views.pdfbota.CPDFBotaFragmentTabs;
import com.compdfkit.tools.common.views.pdfview.CPreviewMode;
import com.compdfkit.tools.viewer.pdfsearch.CSearchResultDialogFragment;
import com.compdfkit.ui.proxy.form.CPDFComboboxWidgetImpl;
import com.compdfkit.ui.proxy.form.CPDFListboxWidgetImpl;
import com.compdfkit.ui.proxy.form.CPDFPushbuttonWidgetImpl;
import com.compdfkit.ui.reader.CPDFReaderView;

import java.util.ArrayList;

import pub.devrel.easypermissions.EasyPermissions;

public class PDFFormSampleActivity extends CBasicActivity {

    /**
     * assets folder pdf file
     */
    public static final String QUICK_START_GUIDE = "Quick Start Guide V2.4.0.pdf";

    private FormPdfSampleActivityBinding binding;

    CFormSampleScreenManager screenManager = new CFormSampleScreenManager();

    private ActivityResultLauncher<Intent> selectDocumentLauncher = registerForActivityResult(new ActivityResultContracts.StartActivityForResult(), result -> {
        if (result.getData() != null && result.getData().getData() != null) {
            Uri uri = result.getData().getData();
            CFileUtils.takeUriPermission(this, uri);
            binding.pdfView.resetFormType();
            binding.formToolBar.reset();
            binding.pdfView.openPDF(uri);
        }
    });

    @Override
    protected void onCreate(Bundle savedInstanceState) {
        super.onCreate(savedInstanceState);
        binding = FormPdfSampleActivityBinding.inflate(getLayoutInflater());
        setContentView(binding.getRoot());
        screenManager.bind(binding);
        //Extract PDF files from the Android assets folder
        initPDFView();
        initToolBarView();
        initSearchBar();
        initFormToolbar();
    }

    private void initPDFView() {
        //Open the PDF file from the assets folder
        CExtractAssetFileTask.extract(this, QUICK_START_GUIDE, QUICK_START_GUIDE, (filePath) ->
                binding.pdfView.openPDF(filePath));
        //Set default properties for PDF forms, such as background color, font color, font size, and other attributes.
        initFormAttr(binding.pdfView);
        binding.pdfView.getCPdfReaderView().setMinScaleEnable(false);
        Paint paint = new Paint();
        paint.setStyle(Paint.Style.FILL);
        paint.setColor(ContextCompat.getColor(this, R.color.tools_color_accent_50));
        binding.pdfView.getCPdfReaderView().setFormPreviewPaint(paint);
        binding.pdfView.getCPdfReaderView().setCurrentFocusedFormType(CPDFWidget.WidgetType.Widget_Unknown);
        binding.pdfView.getCPdfReaderView().setCurrentFocusedType(CPDFAnnotation.Type.WIDGET);
        binding.pdfView.getCPdfReaderView().setTouchMode(CPDFReaderView.TouchMode.BROWSE);
        binding.pdfView.setAllowAddAndEditAnnot(true);
        resetContextMenu(binding.pdfView, CPreviewMode.Form);
        binding.pdfView.setOnTapMainDocAreaCallback(() -> {
            //Use the CFillScreenManager.class to manage fullscreen switching.
            screenManager.fillScreenChange();
        });
        binding.pdfView.getCPdfReaderView().setPdfAddAnnotCallback((cpdfPageView, cpdfBaseAnnot) -> {
            // Annotation creation completed listener, you can use cpdfBaseAnnot.getAnnotType() to determine the type of the added annotation
            if (cpdfBaseAnnot instanceof CPDFListboxWidgetImpl) {
                // When the ListBox form is created, display an editing dialog for adding list data
                CPDFAnnotationManager annotationManager = new CPDFAnnotationManager();
                annotationManager.showFormListEditFragment(getSupportFragmentManager(), cpdfBaseAnnot, cpdfPageView, false);
            } else if (cpdfBaseAnnot instanceof CPDFComboboxWidgetImpl) {
                // When the ComboBox form is created, display an editing dialog for adding list data
                CPDFAnnotationManager annotationManager = new CPDFAnnotationManager();
                annotationManager.showFormComboBoxEditFragment(getSupportFragmentManager(), cpdfBaseAnnot, cpdfPageView,true);
            } else if (cpdfBaseAnnot instanceof CPDFPushbuttonWidgetImpl) {
                // When the PushButton form is created, display a dialog for editing the action method
                CPDFAnnotationManager annotationManager = new CPDFAnnotationManager();
                annotationManager.showPushButtonActionDialog(getSupportFragmentManager(), binding.pdfView.getCPdfReaderView(),
                        cpdfBaseAnnot, cpdfPageView);
            }
        });
        registerFormHelper(binding.pdfView);
    }

    private void initToolBarView() {
        binding.pdfToolBar.addMode(CPreviewMode.Form);
        binding.pdfToolBar.setPreviewModeChangeListener((mode) -> {
            if (mode == CPreviewMode.Viewer){
                binding.pdfView.getCPdfReaderView().setCurrentFocusedFormType(CPDFWidget.WidgetType.Widget_Unknown);
                binding.pdfView.getCPdfReaderView().setCurrentFocusedType(CPDFAnnotation.Type.UNKNOWN);
                binding.pdfView.getCPdfReaderView().setTouchMode(CPDFReaderView.TouchMode.BROWSE);
                binding.pdfView.setAllowAddAndEditAnnot(true);
            }else {
                binding.pdfView.getCPdfReaderView().setCurrentFocusedFormType(CPDFWidget.WidgetType.Widget_Unknown);
                binding.pdfView.getCPdfReaderView().setCurrentFocusedType(CPDFAnnotation.Type.WIDGET);
                binding.pdfView.getCPdfReaderView().setTouchMode(CPDFReaderView.TouchMode.BROWSE);
                binding.pdfView.setAllowAddAndEditAnnot(true);
            }
            screenManager.changeWindowStatus(mode);
            binding.pdfToolBar.selectMode(mode);
            binding.formToolBar.reset();
            resetContextMenu(binding.pdfView, mode);
        });
        binding.pdfToolBar.setSearchBtnClickListener(v -> {
            binding.pdfToolBar.setVisibility(View.GONE);
            binding.pdfSearchToolBar.setVisibility(View.VISIBLE);
            binding.pdfSearchToolBar.showKeyboard();
        });
        binding.pdfToolBar.setThumbnailBtnClickListener(v -> {
            CPDFBotaDialogFragment dialogFragment = CPDFBotaDialogFragment.newInstance();
            dialogFragment.initWithPDFView(binding.pdfView);
            CPDFBotaFragmentTabs annotationTab = new CPDFBotaFragmentTabs(CPDFBOTA.THUMBNAIL, getString(R.string.tools_thumbnails));
            dialogFragment.setBotaDialogTab(annotationTab);
            dialogFragment.show(getSupportFragmentManager(), "ThumbnailList");
        });
        binding.pdfToolBar.setBoTaBtnClickListener(v -> {
            ArrayList<CPDFBotaFragmentTabs> tabs = new ArrayList<>();
            CPDFBotaFragmentTabs annotationTab = new CPDFBotaFragmentTabs(CPDFBOTA.ANNOTATION, getString(R.string.tools_annotations));
            CPDFBotaFragmentTabs outlineTab = new CPDFBotaFragmentTabs(CPDFBOTA.OUTLINE, getString(R.string.tools_outlines));
            CPDFBotaFragmentTabs bookmarkTab = new CPDFBotaFragmentTabs(CPDFBOTA.BOOKMARKS, getString(R.string.tools_bookmarks));
            if (binding.pdfToolBar.getMode() == CPreviewMode.Viewer) {
                tabs.add(outlineTab);
                tabs.add(bookmarkTab);
            } else {
                tabs.add(outlineTab);
                tabs.add(bookmarkTab);
                tabs.add(annotationTab);
            }
            CPDFBotaDialogFragment dialogFragment = CPDFBotaDialogFragment.newInstance();
            dialogFragment.initWithPDFView(binding.pdfView);
            dialogFragment.setBotaDialogTabs(tabs);
            dialogFragment.show(getSupportFragmentManager(), "annotationList");
        });
        binding.pdfToolBar.setMoreBtnClickListener(v -> {
            v.setSelected(true);
            //Show the PDF settings dialog fragment
            CPopupMenuWindow menuWindow = new CPopupMenuWindow(this);
            menuWindow.addItem(R.drawable.tools_ic_preview_settings, R.string.tools_view_setting, v1 -> {
                showDisplaySettings(binding.pdfView);
            });
            menuWindow.addItem(R.drawable.tools_ic_document_info, R.string.tools_document_info, v1 -> {
                showDocumentInfo(binding.pdfView);
            });
            menuWindow.addItem(R.drawable.tools_ic_share, R.string.tools_share, v1 -> {
                sharePDF(binding.pdfView);
            });
            menuWindow.addItem(R.drawable.tools_ic_new_file, R.string.tools_open_document, v1 -> {
                if (hasPermissions(STORAGE_PERMISSIONS)) {
                    selectDocument();
                } else {
                    EasyPermissions.requestPermissions(this, "request permission", REQUEST_EXTERNAL_PERMISSION, STORAGE_PERMISSIONS);
                }
            });
            menuWindow.setOnDismissListener(() -> v.setSelected(false));
            menuWindow.showAsDropDown(v);
        });
    }

    private void initFormToolbar() {
        binding.formToolBar.initWithPDFView(binding.pdfView);
        binding.formToolBar.setFragmentManager(getSupportFragmentManager());
    }

    private void initSearchBar() {
        binding.pdfSearchToolBar.initWithPDFView(binding.pdfView);
        binding.pdfSearchToolBar.onSearchQueryResults(list -> {
            CSearchResultDialogFragment searchResultDialog = new CSearchResultDialogFragment();
            searchResultDialog.show(getSupportFragmentManager(), "searchResultDialogFragment");
            searchResultDialog.setSearchTextInfos(list);
            searchResultDialog.setOnClickSearchItemListener(clickItem -> {
                binding.pdfView.getCPdfReaderView().setDisplayPageIndex(clickItem.page);
                binding.pdfView.getCPdfReaderView().getTextSearcher().searchBegin(clickItem.page, clickItem.textRangeIndex);
                searchResultDialog.dismiss();
            });
        });
        binding.pdfSearchToolBar.setExitSearchListener(() -> {
            binding.pdfToolBar.setVisibility(View.VISIBLE);
            binding.pdfSearchToolBar.setVisibility(View.GONE);
        });
    }

    private void selectDocument() {
        if (!binding.pdfView.getCPdfReaderView().getPDFDocument().hasChanges()) {
            selectDocumentLauncher.launch(CFileUtils.getContentIntent());
            return;
        }
        CAlertDialog alertDialog = CAlertDialog.newInstance(getString(com.compdfkit.tools.R.string.tools_save_title), getString(com.compdfkit.tools.R.string.tools_save_message));
        alertDialog.setConfirmClickListener(v -> {
            //save pdf document
            binding.pdfView.savePDF((filePath, pdfUri) -> {
                alertDialog.dismiss();
                selectDocumentLauncher.launch(CFileUtils.getContentIntent());
            });
        });
        alertDialog.setCancelClickListener(v -> {
            alertDialog.dismiss();
            selectDocumentLauncher.launch(CFileUtils.getContentIntent());
        });
        alertDialog.show(getSupportFragmentManager(), "alertDialog");
    }

    @Override
    public void onRequestPermissionsResult(int requestCode, @NonNull String[] permissions, @NonNull int[] grantResults) {
        super.onRequestPermissionsResult(requestCode, permissions, grantResults);
        if (requestCode == REQUEST_EXTERNAL_PERMISSION) {
            if (EasyPermissions.hasPermissions(this, STORAGE_PERMISSIONS)) {
                selectDocument();
            }
        }
    }

    @Override
    protected void onSaveInstanceState(@NonNull Bundle outState) {
        binding.pdfView.savePDF(null);
        super.onSaveInstanceState(outState);
    }

    @Override
    public void onBackPressed() {
        if (binding.pdfView != null) {
            binding.pdfView.savePDF((filePath, pdfUri) -> super.onBackPressed());
        }
    }
}

==> An_V2.0.3.tar.gz: kmpdfkit_demo/Forms/src/main/java/com/compdfkit/forms/PDFFormSampleActivity.java <==
package com.compdfkit.forms;

import android.Manifest;
import android.graphics.Paint;
import android.os.Build;
import android.os.Bundle;
import android.view.View;

import androidx.activity.result.ActivityResultLauncher;
import androidx.annotation.NonNull;
import androidx.core.content.ContextCompat;

import com.compdfkit.forms.databinding.FormPdfSampleActivityBinding;
import com.compdfkit.tools.common.basic.activity.CBasicPDFActivity;
import com.compdfkit.tools.common.pdf.config.FormsConfig;
import com.compdfkit.tools.common.utils.CFileUtils;
import com.compdfkit.tools.common.utils.CPermissionUtil;
import com.compdfkit.tools.common.utils.activitycontracts.CSelectPDFDocumentResultContract;
import com.compdfkit.tools.common.utils.annotation.CPDFAnnotationManager;
import com.compdfkit.tools.common.utils.dialog.CAlertDialog;
import com.compdfkit.tools.common.utils.task.CExtractAssetFileTask;
import com.compdfkit.tools.common.utils.window.CPopupMenuWindow;
import com.compdfkit.tools.common.views.pdfbota.CPDFBOTA;
import com.compdfkit.tools.common.views.pdfbota.CPDFBotaDialogFragment;
import com.compdfkit.tools.common.views.pdfbota.CPDFBotaFragmentTabs;
import com.compdfkit.tools.common.views.pdfview.CPreviewMode;
import com.compdfkit.tools.viewer.pdfsearch.CSearchResultDialogFragment;
import com.compdfkit.ui.proxy.form.CPDFComboboxWidgetImpl;
import com.compdfkit.ui.proxy.form.CPDFListboxWidgetImpl;
import com.compdfkit.ui.proxy.form.CPDFPushbuttonWidgetImpl;
import com.compdfkit.ui.reader.CPDFReaderView;

import java.util.ArrayList;
import java.util.Arrays;

public class PDFFormSampleActivity extends CBasicPDFActivity {

    /**
     * assets folder pdf file
     */
    public static final String QUICK_START_GUIDE = "ComPDFKit_Forms_Sample_File.pdf";

    private FormPdfSampleActivityBinding binding;

    CFormSampleScreenManager screenManager = new CFormSampleScreenManager();

    private ActivityResultLauncher<Void> selectDocumentLauncher = registerForActivityResult(new CSelectPDFDocumentResultContract(), uri -> {
        if (uri != null) {
            CFileUtils.takeUriPermission(this, uri);
            binding.pdfView.resetFormType();
            binding.formToolBar.reset();
            binding.pdfView.openPDF(uri);
        }
    });

    @Override
    protected void onCreate(Bundle savedInstanceState) {
        super.onCreate(savedInstanceState);
        binding = FormPdfSampleActivityBinding.inflate(getLayoutInflater());
        setContentView(binding.getRoot());
        screenManager.bind(binding);
        //Extract PDF files from the Android assets folder
        initPDFView();
        initToolBarView();
        initSearchBar();
        initFormToolbar();
    }

    private void initPDFView() {
        //Open the PDF file from the assets folder
        CExtractAssetFileTask.extract(this, QUICK_START_GUIDE, QUICK_START_GUIDE, (filePath) ->
                binding.pdfView.openPDF(filePath));
        //Set default properties for PDF forms, such as background color, font color, font size, and other attributes.
        initFormAttr(binding.pdfView);
        binding.pdfView.getCPdfReaderView().setMinScaleEnable(false);
        Paint paint = new Paint();
        paint.setStyle(Paint.Style.FILL);
        paint.setColor(ContextCompat.getColor(this, R.color.tools_color_accent_50));
        binding.pdfView.getCPdfReaderView().setFormPreviewPaint(paint);
        binding.pdfView.setViewMode(CPDFReaderView.ViewMode.FORM);
        resetContextMenu(binding.pdfView, CPreviewMode.Form);
        binding.pdfView.setOnTapMainDocAreaCallback(() -> {
            //Use the CFillScreenManager.class to manage fullscreen switching.
            screenManager.fillScreenChange();
        });
        binding.pdfView.getCPdfReaderView().setPdfAddAnnotCallback((cpdfPageView, cpdfBaseAnnot) -> {
            // Annotation creation completed listener, you can use cpdfBaseAnnot.getAnnotType() to determine the type of the added annotation
            if (cpdfBaseAnnot instanceof CPDFListboxWidgetImpl) {
                // When the ListBox form is created, display an editing dialog for adding list data
                CPDFAnnotationManager annotationManager = new CPDFAnnotationManager();
                annotationManager.showFormListEditFragment(getSupportFragmentManager(), cpdfBaseAnnot, cpdfPageView, false);
            } else if (cpdfBaseAnnot instanceof CPDFComboboxWidgetImpl) {
                // When the ComboBox form is created, display an editing dialog for adding list data
                CPDFAnnotationManager annotationManager = new CPDFAnnotationManager();
                annotationManager.showFormComboBoxEditFragment(getSupportFragmentManager(), cpdfBaseAnnot, cpdfPageView, true);
            } else if (cpdfBaseAnnot instanceof CPDFPushbuttonWidgetImpl) {
                // When the PushButton form is created, display a dialog for editing the action method
                CPDFAnnotationManager annotationManager = new CPDFAnnotationManager();
                annotationManager.showPushButtonActionDialog(getSupportFragmentManager(), binding.pdfView.getCPdfReaderView(),
                        cpdfBaseAnnot, cpdfPageView);
            }
        });
        registerFormHelper(binding.pdfView);
    }

    private void initToolBarView() {
        binding.pdfToolBar.addModes(Arrays.asList(CPreviewMode.Viewer,
                CPreviewMode.Form));
        binding.pdfToolBar.selectMode(CPreviewMode.Form);
        binding.pdfToolBar.setPreviewModeChangeListener((mode) -> {
            if (mode == CPreviewMode.Viewer) {
                binding.pdfView.setViewMode(CPDFReaderView.ViewMode.VIEW);
            } else {
                binding.pdfView.setViewMode(CPDFReaderView.ViewMode.FORM);
            }
            screenManager.changeWindowStatus(mode);
            binding.pdfToolBar.selectMode(mode);
            binding.formToolBar.reset();
            resetContextMenu(binding.pdfView, mode);
        });

        binding.pdfToolBar.addAction(R.drawable.tools_ic_thumbnail, v -> {

            CPDFBotaDialogFragment dialogFragment = CPDFBotaDialogFragment.newInstance();
            dialogFragment.initWithPDFView(binding.pdfView);
            CPDFBotaFragmentTabs annotationTab = new CPDFBotaFragmentTabs(CPDFBOTA.THUMBNAIL, getString(R.string.tools_thumbnails));
            dialogFragment.setBotaDialogTab(annotationTab);
            dialogFragment.show(getSupportFragmentManager(), "ThumbnailList");
        });
        binding.pdfToolBar.addAction(R.drawable.tools_ic_search, v -> {
            binding.pdfToolBar.setVisibility(View.GONE);
            binding.pdfSearchToolBar.setVisibility(View.VISIBLE);
            binding.pdfSearchToolBar.showKeyboard();
        });

        binding.pdfToolBar.addAction(R.drawable.tools_ic_bookmark, v -> {
            ArrayList<CPDFBotaFragmentTabs> tabs = new ArrayList<>();
            CPDFBotaFragmentTabs annotationTab = new CPDFBotaFragmentTabs(CPDFBOTA.ANNOTATION, getString(R.string.tools_annotations));
            CPDFBotaFragmentTabs outlineTab = new CPDFBotaFragmentTabs(CPDFBOTA.OUTLINE, getString(R.string.tools_outlines));
            CPDFBotaFragmentTabs bookmarkTab = new CPDFBotaFragmentTabs(CPDFBOTA.BOOKMARKS, getString(R.string.tools_bookmarks));
            if (binding.pdfToolBar.getMode() == CPreviewMode.Viewer) {
                tabs.add(outlineTab);
                tabs.add(bookmarkTab);
            } else {
                tabs.add(outlineTab);
                tabs.add(bookmarkTab);
                tabs.add(annotationTab);
            }
            CPDFBotaDialogFragment dialogFragment = CPDFBotaDialogFragment.newInstance();
            dialogFragment.initWithPDFView(binding.pdfView);
            dialogFragment.setBotaDialogTabs(tabs);
            dialogFragment.show(getSupportFragmentManager(), "annotationList");
        });
        binding.pdfToolBar.addAction(R.drawable.tools_ic_more, v -> {
            v.setSelected(true);
            //Show the PDF settings dialog fragment
            CPopupMenuWindow menuWindow = new CPopupMenuWindow(this);
            menuWindow.addItem(R.drawable.tools_ic_preview_settings, R.string.tools_view_setting, v1 -> {
                showDisplaySettings(binding.pdfView);
            });
            menuWindow.addItem(R.drawable.tools_ic_document_info, R.string.tools_document_info, v1 -> {
                showDocumentInfo(binding.pdfView);
            });
            menuWindow.addItem(R.drawable.tools_ic_share, R.string.tools_share, v1 -> {
                sharePDF(binding.pdfView);
            });
            menuWindow.addItem(R.drawable.tools_ic_new_file, R.string.tools_open_document, v1 -> {
                if (CPermissionUtil.hasStoragePermissions(this)) {
                    selectDocument();
                } else {
                    requestStoragePermissions();
                }
            });
            menuWindow.setOnDismissListener(() -> v.setSelected(false));
            menuWindow.showAsDropDown(v);
        });
    }

    private void requestStoragePermissions() {
        if (Build.VERSION.SDK_INT >= Build.VERSION_CODES.TIRAMISU) {
            CPermissionUtil.openManageAllFileAppSettings(this);
        } else {
            multiplePermissionResultLauncher.launch(STORAGE_PERMISSIONS, result -> {
                if (CPermissionUtil.hasStoragePermissions(this)) {
                    selectDocument();
                } else {
                    if (!CPermissionUtil.shouldShowRequestPermissionRationale(this, Manifest.permission.WRITE_EXTERNAL_STORAGE)) {
                        showPermissionsRequiredDialog();
                    }
                }
            });
        }
    }

    private void initFormToolbar() {
        binding.formToolBar.initWithPDFView(binding.pdfView);
        binding.formToolBar.setTools(Arrays.asList(
                FormsConfig.FormsTools.Undo,
                FormsConfig.FormsTools.Redo
        ));
        binding.formToolBar.setFragmentManager(getSupportFragmentManager());
    }

    private void initSearchBar() {
        binding.pdfSearchToolBar.initWithPDFView(binding.pdfView);
        binding.pdfSearchToolBar.onSearchQueryResults(list -> {
            CSearchResultDialogFragment searchResultDialog = new CSearchResultDialogFragment();
            searchResultDialog.show(getSupportFragmentManager(), "searchResultDialogFragment");
            searchResultDialog.setSearchTextInfos(list);
            searchResultDialog.setOnClickSearchItemListener(clickItem -> {
                binding.pdfView.getCPdfReaderView().setDisplayPageIndex(clickItem.page);
                binding.pdfView.getCPdfReaderView().getTextSearcher().searchBegin(clickItem.page, clickItem.textRangeIndex);
                searchResultDialog.dismiss();
            });
        });
        binding.pdfSearchToolBar.setExitSearchListener(() -> {
            binding.pdfToolBar.setVisibility(View.VISIBLE);
            binding.pdfSearchToolBar.setVisibility(View.GONE);
        });
        binding.pdfSearchToolBar.enableSearchResultList(false);
    }

    private void selectDocument() {
        if (binding.pdfView.getCPdfReaderView().getPDFDocument() == null) {
            selectDocumentLauncher.launch(null);
            return;
        }
        if (!binding.pdfView.getCPdfReaderView().getPDFDocument().hasChanges()) {
            selectDocumentLauncher.launch(null);
            return;
        }
        CAlertDialog alertDialog = CAlertDialog.newInstance(getString(R.string.tools_save_title), getString(R.string.tools_save_message));
        alertDialog.setConfirmClickListener(v -> {
            //save pdf document
            binding.pdfView.savePDF((filePath, pdfUri) -> {
                alertDialog.dismiss();
                selectDocumentLauncher.launch(null);
            }, e -> {
                alertDialog.dismiss();
                selectDocumentLauncher.launch(null);
            });
        });
        alertDialog.setCancelClickListener(v -> {
            alertDialog.dismiss();
            selectDocumentLauncher.launch(null);
        });
        alertDialog.show(getSupportFragmentManager(), "alertDialog");
    }

    @Override
    protected void onSaveInstanceState(@NonNull Bundle outState) {
        binding.pdfView.savePDF(null, e -> {
        });
        super.onSaveInstanceState(outState);
    }

    @Override
    public void onBackPressed() {
        if (binding.pdfView != null) {
            binding.pdfView.savePDF((filePath, pdfUri) -> super.onBackPressed(), e -> super.onBackPressed());
        } else {
            super.onBackPressed();
        }
    }
}
